using System;

namespace ComboGDBG
{
	using System;
	using System.Drawing;
	using System.Diagnostics;
	using System.Collections;
	using System.ComponentModel;
	using System.Windows.Forms;
	using System.Runtime.InteropServices;

	public class Underline
	{
		public Underline(RichTextBox RTB)
		{
			this.R = RTB;
		}
		private const int CFM_UNDERLINETYPE = 8388608;
		private const int EM_SETCHARFORMAT = 1092;
		private const int EM_GETCHARFORMAT = 1082;
		private const int  SCF_SELECTION = 1;
		private RichTextBox R;

		[StructLayout(LayoutKind.Sequential)]
			private struct CHARFORMAT
		{
			public int cbSize;
			public uint dwMask;
			public uint dwEffects;
			public int yHeight;
			public int yOffset;
			public int crTextColor;
			public byte bCharSet;
			public byte bPitchAndFamily;
			[MarshalAs(UnmanagedType.ByValArray, SizeConst = 32)]
			public char[] szFaceName;

			// CHARFORMAT2 from here onwards. 
			public short wWeight;
			public short sSpacing;
			public int crBackColor;
			public int LCID;
			public uint dwReserved;
			public short sStyle;
			public short wKerning;
			public byte bUnderlineType;
			public byte bAnimation;
			public byte bRevAuthor;
		}

		[DllImport("user32", CharSet = CharSet.Auto)]
		private static extern int SendMessage(HandleRef hWnd, int msg, int wParam, ref CHARFORMAT lp);

		public UnderlineStyle SelectionUnderlineStyle 
		{
			get 
			{
				CHARFORMAT fmt = new CHARFORMAT();
				fmt.cbSize = Marshal.SizeOf(fmt);

				// Get the underline style. 
				SendMessage(new HandleRef(this, R.Handle), EM_GETCHARFORMAT, SCF_SELECTION,ref fmt);

				// Default to no underline. 
				if ((fmt.dwMask & CFM_UNDERLINETYPE) == 0)
				{
					return UnderlineStyle.None;
				}

				byte style = Convert.ToByte( fmt.bUnderlineType & 15);

				return (UnderlineStyle)style;
			}

			set 
			{
				// Ensure we don't alter the color by accident. 
				UnderlineColor color = SelectionUnderlineColor;

				// Ensure we don't show it if it shouldn't be shown. 
				if (value == UnderlineStyle.None)
				{
					color = UnderlineColor.Black;
				}

				CHARFORMAT fmt = new CHARFORMAT();
				fmt.cbSize = Marshal.SizeOf(fmt);
				fmt.dwMask = CFM_UNDERLINETYPE;
				fmt.bUnderlineType = Convert.ToByte( Convert.ToByte(value) |Convert.ToByte(color));

				// Set the underline type. 
				SendMessage(new HandleRef(this, R.Handle), EM_SETCHARFORMAT, SCF_SELECTION, ref fmt);
			}
		}

		public enum UnderlineStyle
		{
			/// <summary> 
			/// No underlining. 
			/// </summary> 
			None = 0,

			/// <summary> 
			/// Standard underlining across all words. 
			/// </summary> 
			Normal = 1,

			/// <summary> 
			/// Standard underlining broken between words. 
			/// </summary> 
			Word = 2,

			/// <summary> 
			/// Double line underlining. 
			/// </summary> 
			Double = 3,

			/// <summary> 
			/// Dotted underlining. 
			/// </summary> 
			Dotted = 4,

			/// <summary> 
			/// Dashed underlining. 
			/// </summary> 
			Dash = 5,

			/// <summary> 
			/// Dash-dot ("-.-.") underlining. 
			/// </summary> 
			DashDot = 6,

			/// <summary> 
			/// Dash-dot-dot ("-..-..") underlining. 
			/// </summary> 
			DashDotDot = 7,

			/// <summary> 
			/// Wave underlining (like spelling mistakes in MS Word). 
			/// </summary> 
			Wave = 8,

			/// <summary> 
			/// Extra thick standard underlining. 
			/// </summary> 
			Thick = 9,

			/// <summary> 
			/// Extra thin standard underlining. 
			/// </summary> 
			HairLine = 10,

			/// <summary> 
			/// Double thickness wave underlining. 
			/// </summary> 
			DoubleWave = 11,

			/// <summary> 
			/// Thick wave underlining. 
			/// </summary> 
			HeavyWave = 12,

			/// <summary> 
			/// Extra long dash underlining. 
			/// </summary> 
			LongDash = 13
		}

		public UnderlineColor SelectionUnderlineColor 
		{
			get 
			{
				CHARFORMAT fmt = new CHARFORMAT();
				fmt.cbSize = Marshal.SizeOf(fmt);

				// Get the underline color. 
				SendMessage(new HandleRef(this, R.Handle), EM_GETCHARFORMAT, SCF_SELECTION, ref fmt);

				// Default to black. 
				if ((fmt.dwMask & CFM_UNDERLINETYPE) == 0)
				{
					return UnderlineColor.Black;
				}

				byte style = Convert.ToByte(fmt.bUnderlineType & 240);

				return (UnderlineColor)style;
			}

			set 
			{
				// Ensure we don't alter the style. 
				UnderlineStyle style = SelectionUnderlineStyle;

				// Ensure we don't show it if it shouldn't be shown. 
				if (style == UnderlineStyle.None)
				{
					value = UnderlineColor.Black;
				}

				CHARFORMAT fmt = new CHARFORMAT();
				fmt.cbSize = Marshal.SizeOf(fmt);
				fmt.dwMask = CFM_UNDERLINETYPE;
				fmt.bUnderlineType = Convert.ToByte(Convert.ToByte(style) | Convert.ToByte(value));

				// Set the underline color. 
				SendMessage(new HandleRef(this, R.Handle), EM_SETCHARFORMAT, SCF_SELECTION,ref  fmt);
			}
		}

		public enum UnderlineColor
		{
			/// <summary>Black.</summary> 
			Black = 0,

			/// <summary>Blue.</summary> 
			Blue = 16,

			/// <summary>Cyan.</summary> 
			Cyan = 32,

			/// <summary>Lime green.</summary> 
			LimeGreen = 48,

			/// <summary>Magenta.</summary> 
			Magenta = 64,

			/// <summary>Red.</summary> 
			Red = 80,

			/// <summary>Yellow.</summary> 
			Yellow = 96,

			/// <summary>White.</summary> 
			White = 112,

			/// <summary>DarkBlue.</summary> 
			DarkBlue = 128,

			/// <summary>DarkCyan.</summary> 
			DarkCyan = 144,

			/// <summary>Green.</summary> 
			Green = 160,

			/// <summary>Dark magenta.</summary> 
			DarkMagenta = 176,

			/// <summary>Brown.</summary> 
			Brown = 192,

			/// <summary>Olive green.</summary> 
			OliveGreen = 208,

			/// <summary>Dark gray.</summary> 
			DarkGray = 224,

			/// <summary>Gray.</summary> 
			Gray = 240
		}
	}
}
